@@ -38,9 +38,12 @@
 #define OUTPUT_UNITS P->right
 
 
-static PJ_COORD fwd_prepare (PJ *P, PJ_COORD coo) {
+static void fwd_prepare (PJ *P, PJ_COORD& coo) {
     if (HUGE_VAL==coo.v[0] || HUGE_VAL==coo.v[1] || HUGE_VAL==coo.v[2])
-        return proj_coord_error ();
+    {
+        coo = proj_coord_error ();
+        return;
+    }
 
     /* The helmert datum shift will choke unless it gets a sensible 4D coordinate */
     if (HUGE_VAL==coo.v[2] && P->helmert) coo.v[2] = 0.0;
@@ -52,10 +55,21 @@ static PJ_COORD fwd_prepare (PJ *P, PJ_COORD coo) {
 
         /* check for latitude or longitude over-range */
         t = (coo.lp.phi < 0  ?  -coo.lp.phi  :  coo.lp.phi) - M_HALFPI;
-        if (t > PJ_EPS_LAT  ||  coo.lp.lam > 10  ||  coo.lp.lam < -10) {
-            proj_errno_set (P, PJD_ERR_LAT_OR_LON_EXCEED_LIMIT);
-            return proj_coord_error ();
+        if (t > PJ_EPS_LAT)
+        {
+            proj_log_error(P, _("Invalid latitude"));
+            proj_errno_set (P, PROJ_ERR_COORD_TRANSFM_INVALID_COORD);
+            coo = proj_coord_error ();
+            return;
         }
+        if (coo.lp.lam > 10  ||  coo.lp.lam < -10)
+        {
+            proj_log_error(P, _("Invalid longitude"));
+            proj_errno_set (P, PROJ_ERR_COORD_TRANSFM_INVALID_COORD);
+            coo = proj_coord_error ();
+            return;
+        }
+
 
         /* Clamp latitude to -90..90 degree range */
         if (coo.lp.phi > M_HALFPI)
@@ -80,7 +94,7 @@ static PJ_COORD fwd_prepare (PJ *P, PJ_COORD coo) {
             coo = proj_trans (P->cart,       PJ_INV, coo); /* Go back to angular using local ellps */
         }
         if (coo.lp.lam==HUGE_VAL)
-            return coo;
+            return;
         if (P->vgridshift)
             coo = proj_trans (P->vgridshift, PJ_FWD, coo); /* Go orthometric from geometric */
 
@@ -91,18 +105,18 @@ static PJ_COORD fwd_prepare (PJ *P, PJ_COORD coo) {
         if (0==P->over)
             coo.lp.lam = adjlon(coo.lp.lam);
 
-        return coo;
+        return;
     }
 
 
     /* We do not support gridshifts on cartesian input */
     if (INPUT_UNITS==PJ_IO_UNITS_CARTESIAN && P->helmert)
-            return proj_trans (P->helmert, PJ_INV, coo);
-    return coo;
+            coo = proj_trans (P->helmert, PJ_INV, coo);
+    return;
 }
 
 
-static PJ_COORD fwd_finalize (PJ *P, PJ_COORD coo) {
+static void fwd_finalize (PJ *P, PJ_COORD& coo) {
 
     switch (OUTPUT_UNITS) {
 
@@ -134,6 +148,9 @@ static PJ_COORD fwd_finalize (PJ *P, PJ_COORD coo) {
     case PJ_IO_UNITS_WHATEVER:
         break;
 
+    case PJ_IO_UNITS_DEGREES:
+        break;
+
     case PJ_IO_UNITS_RADIANS:
         coo.lpz.z = P->vfr_meter * (coo.lpz.z + P->z0);
 
@@ -149,29 +166,28 @@ static PJ_COORD fwd_finalize (PJ *P, PJ_COORD coo) {
 
     if (P->axisswap)
         coo = proj_trans (P->axisswap, PJ_FWD, coo);
-
-    return coo;
 }
 
 
-static PJ_COORD error_or_coord(PJ *P, PJ_COORD coord, int last_errno) {
-    if (proj_errno(P))
+static inline PJ_COORD error_or_coord(PJ *P, PJ_COORD coord, int last_errno) {
+    if (P->ctx->last_errno)
         return proj_coord_error();
 
-    proj_errno_restore(P, last_errno);
+    P->ctx->last_errno = last_errno;
+
     return coord;
 }
 
 
 PJ_XY pj_fwd(PJ_LP lp, PJ *P) {
-    int last_errno;
     PJ_COORD coo = {{0,0,0,0}};
     coo.lp = lp;
 
-    last_errno = proj_errno_reset(P);
+    const int last_errno = P->ctx->last_errno;
+    P->ctx->last_errno = 0;
 
     if (!P->skip_fwd_prepare)
-        coo = fwd_prepare (P, coo);
+        fwd_prepare (P, coo);
     if (HUGE_VAL==coo.v[0] || HUGE_VAL==coo.v[1])
         return proj_coord_error ().xy;
 
@@ -183,14 +199,14 @@ PJ_XY pj_fwd(PJ_LP lp, PJ *P) {
     else if (P->fwd4d)
         coo = P->fwd4d (coo, P);
     else {
-        proj_errno_set (P, EINVAL);
+        proj_errno_set (P, PROJ_ERR_OTHER_NO_INVERSE_OP);
         return proj_coord_error ().xy;
     }
     if (HUGE_VAL==coo.v[0])
         return proj_coord_error ().xy;
 
     if (!P->skip_fwd_finalize)
-        coo = fwd_finalize (P, coo);
+        fwd_finalize (P, coo);
 
     return error_or_coord(P, coo, last_errno).xy;
 }
@@ -198,14 +214,14 @@ PJ_XY pj_fwd(PJ_LP lp, PJ *P) {
 
 
 PJ_XYZ pj_fwd3d(PJ_LPZ lpz, PJ *P) {
-    int last_errno;
     PJ_COORD coo = {{0,0,0,0}};
     coo.lpz = lpz;
 
-    last_errno = proj_errno_reset(P);
+    const int last_errno = P->ctx->last_errno;
+    P->ctx->last_errno = 0;
 
     if (!P->skip_fwd_prepare)
-        coo = fwd_prepare (P, coo);
+        fwd_prepare (P, coo);
     if (HUGE_VAL==coo.v[0])
         return proj_coord_error ().xyz;
 
@@ -217,14 +233,14 @@ PJ_XYZ pj_fwd3d(PJ_LPZ lpz, PJ *P) {
     else if (P->fwd)
         coo.xy = P->fwd (coo.lp, P);
     else {
-        proj_errno_set (P, EINVAL);
+        proj_errno_set (P, PROJ_ERR_OTHER_NO_INVERSE_OP);
         return proj_coord_error ().xyz;
     }
     if (HUGE_VAL==coo.v[0])
         return proj_coord_error ().xyz;
 
     if (!P->skip_fwd_finalize)
-        coo = fwd_finalize (P, coo);
+        fwd_finalize (P, coo);
 
     return error_or_coord(P, coo, last_errno).xyz;
 }
@@ -232,10 +248,12 @@ PJ_XYZ pj_fwd3d(PJ_LPZ lpz, PJ *P) {
 
 
 PJ_COORD pj_fwd4d (PJ_COORD coo, PJ *P) {
-    int last_errno = proj_errno_reset(P);
+
+    const int last_errno = P->ctx->last_errno;
+    P->ctx->last_errno = 0;
 
     if (!P->skip_fwd_prepare)
-        coo = fwd_prepare (P, coo);
+        fwd_prepare (P, coo);
     if (HUGE_VAL==coo.v[0])
         return proj_coord_error ();
 
@@ -247,14 +265,14 @@ PJ_COORD pj_fwd4d (PJ_COORD coo, PJ *P) {
     else if (P->fwd)
         coo.xy  =  P->fwd (coo.lp, P);
     else {
-        proj_errno_set (P, EINVAL);
+        proj_errno_set (P, PROJ_ERR_OTHER_NO_INVERSE_OP);
         return proj_coord_error ();
     }
     if (HUGE_VAL==coo.v[0])
         return proj_coord_error ();
 
     if (!P->skip_fwd_finalize)
-        coo = fwd_finalize (P, coo);
+        fwd_finalize (P, coo);
 
     return error_or_coord(P, coo, last_errno);
 }
